 
// Include the Control Surface library
#include <Control_Surface.h>

// SR04 UlraSonic Sensor
#include <SD.h>
#include "SR04.h"
#define TRIG_PIN 7
#define ECHO_PIN 8
SR04 sr04 = SR04(ECHO_PIN,TRIG_PIN);
long a;
int nonote_count;

int midiMapped = 0; 
int lastmidiMapped = 0; 

MIDIAddress sendNote;

// Instantiate a MIDI over USB interface.
USBMIDI_Interface midi;
 
// Create a new instance of the class `CCPotentiometer`, called `potentiometer`,
// on pin A0, that sends MIDI messages with controller 7 (channel volume)
// on channel 1.
CCPotentiometer joystick_Y {
  A0, {MIDI_CC::Effect_Control_1, CHANNEL_2}
};
 
CCPotentiometer joystick_X {
  A1, {MIDI_CC::Effect_Control_2, CHANNEL_2}
};

CCPotentiometer expression_PED {
  A2, {MIDI_CC::Expression_Controller, CHANNEL_2}
};
// Push button connected between pin 2 and ground.
// Note message is sent when pressed.
Button greenButt {2};
Button jsButt {3};
Button pedalButt {4};
 
 
// MIDI address of the note to send
const MIDIAddress greenNote {MIDI_Notes::C(1), CHANNEL_2};
const MIDIAddress jsNote {MIDI_Notes::D(1), CHANNEL_2};
const MIDIAddress pedalNote {MIDI_Notes::E(1), CHANNEL_2};
// The velocity of the note events
const uint8_t velocity = 0x7F;
// The filtered value read when potentiometer is at the 0% position
constexpr analog_t minimumValue = 255;
// The filtered value read when potentiometer is at the 100% position
constexpr analog_t maximumValue = 16383 - 255;
 
// A mapping function to eliminate the dead zones of the potentiometer:
// Some potentiometers don't output a perfect zero signal when you move them to
// the zero position, they will still output a value of 1 or 2, and the same
// goes for the maximum position.
analog_t mappingFunction(analog_t raw) {
    // make sure that the analog value is between the minimum and maximum
    raw = constrain(raw, minimumValue, maximumValue);
    // map the value from [minimumValue, maximumValue] to [0, 16383]
    analog_t r1;    
    
    r1 = map(raw, minimumValue, maximumValue, 0, 16383);

    return 16383 - r1;

}

analog_t modMapFunction(analog_t raw) {
    // make sure that the analog value is between the minimum and maximum
    raw = constrain(raw, minimumValue, maximumValue);
    // map the value from [minimumValue, maximumValue] to [0, 16383]
    analog_t r1;

    r1 = map(raw, minimumValue, maximumValue, 0, 16383);

    if( r1 > 8090 && r1 < 8290)
    {
      r1 = 5;
    }
    else if(r1 > 8191)
    {
      r1 = 2*((r1 - 8191)*0.95);
    }
    else
    {
      r1 = -1 * 2*(r1 - 8191)*0.95;
    }

    return r1;
    // Note: 16383 = 2¹⁴ - 1 (the maximum value that can be represented by
    // a 14-bit unsigned number
}
 
void setup() {
    greenButt.begin();
    jsButt.begin();
    pedalButt.begin();
    joystick_Y.map(mappingFunction);
    joystick_X.map(modMapFunction);
    expression_PED.map(mappingFunction);
    // Initialize everything
    midi.begin();  
    Control_Surface.begin();
    Serial.begin(115200);

    nonote_count = 0;

    

   //a=sr04.Distance();
   //Serial.print(a);
   //Serial.println("cm");
}
 
void loop() 
{
    // Update the Control Surface (check whether the potentiometer's
    // input has changed since last time, if so, send the new value over MIDI).
    Control_Surface.loop();
    // Use this to find minimumValue and maximumValue: it prints the raw value
    // of the potentiometer, without the mapping function
    //Serial.println(potentiometer.getRawValue());
    //Serial.println(joystick_Y.getRawValue());

    midi.update(); // read and handle or discard MIDI input
 
  greenButt.update();                              // read the button state
  if (greenButt.getState() == Button::Falling)     // if the button is pressed
    midi.sendNoteOn(greenNote, velocity);         // send a note on event
  else if (greenButt.getState() == Button::Rising) // if the button is released
    midi.sendNoteOff(greenNote, velocity);        // send a note off event


  jsButt.update();                              // read the button state
  if (jsButt.getState() == Button::Falling)     // if the button is pressed
    midi.sendNoteOn(jsNote, velocity);         // send a note on event
  else if (jsButt.getState() == Button::Rising) // if the button is released
    midi.sendNoteOff(jsNote, velocity);        // send a note off event

  pedalButt.update();                              // read the button state
  if (pedalButt.getState() == Button::Falling)     // if the button is pressed
    midi.sendNoteOn(pedalNote, velocity);         // send a note on event
  else if (pedalButt.getState() == Button::Rising) // if the button is released
    midi.sendNoteOff(pedalNote, velocity);        // send a note off event
a=sr04.Distance();
a = a/1.5;

if((a > 35) && (nonote_count < 5))
{
   sendNote = {lastmidiMapped, CHANNEL_2};
   midi.sendNoteOff(sendNote, velocity);
   delay(100);  
  nonote_count = nonote_count+1;
  }
  else 
  {
   midiMapped = map(constrain(a, 1, 36), 1, 36, 48, 72); 
   if (lastmidiMapped != midiMapped) 
   { 
      sendNote = {lastmidiMapped, CHANNEL_2};
     midi.sendNoteOff(sendNote, velocity);  
     sendNote = {midiMapped, CHANNEL_2};  
      midi.sendNoteOn(sendNote, velocity);
     delay(50);
     lastmidiMapped = midiMapped;
     nonote_count = 0;
  }
}
}

